				 
//========================================================================
//
//      Role Object Pattern - March, 2000
//      Jorge Andres Diaz Pace
//
//========================================================================

// RolePattern.Java

//========================================================================

import java.util.Vector;
import java.util.Enumeration;

//========================================================================

// Interface Component

// Models a particular key abstraction by defining its interface. 
// Specifies the protocol for adding, removing, testing and querying 
// for role objects.

interface ROPComponent {

    //--------------------------------------------------------------------
	abstract public boolean addRole(String type);
    //--------------------------------------------------------------------
	abstract public boolean hasRole(String type);
    //--------------------------------------------------------------------
	abstract public boolean removeRole(String type);
    //--------------------------------------------------------------------
	abstract public ROPComponentRole getRole(String type);
    //--------------------------------------------------------------------
	abstract public Enumeration getRoles();
    //--------------------------------------------------------------------
	   
} // End ROPComponent

//========================================================================

// Class ROPComponentCore

// Implements the component interface including the role management 
// protocol.
// Create concrete roles instances.
// Manages its role objects.

class ROPComponentCore implements ROPComponent {

    // Instance variables ////////////////////////////////////////////////

	private Vector roles;	// It stores the different roles played by
							// the component core.

    // Constructor ///////////////////////////////////////////////////////

    //--------------------------------------------------------------------
	public ROPComponentCore() {
		roles = new Vector();
    }
    //--------------------------------------------------------------------

    // Instance methods //////////////////////////////////////////////////
    
	//--------------------------------------------------------------------
	public boolean addRole(String type) {
		if (this.hasRole(type))	return (false);
		ROPComponentRole role = null;
		try {
			try {		// Creates a new concrete role for the component core,
				try {	// according to an specific type
					role = (ROPComponentRole)(Class.forName(type).newInstance());
				} catch (ClassNotFoundException e1)		{ e1.printStackTrace();}
			} catch (InstantiationException e2)			{ e2.printStackTrace();}
		} catch (IllegalAccessException e3)				{ e3.printStackTrace();}
		role.setTarget(this);
		role.setName(type);
		roles.addElement(role);
		return (true);
	}
    //--------------------------------------------------------------------
	public boolean hasRole(String type) {
		ROPComponentRole current;
		for (Enumeration e = roles.elements(); e.hasMoreElements(); ) {
			current = (ROPComponentRole)(e.nextElement());
			if (current.getName().compareTo(type) == 0)	return (true);
		}
		return (false);
	}
    //--------------------------------------------------------------------
	public boolean removeRole(String type) {
		ROPComponentRole current;
		for (Enumeration e = roles.elements(); e.hasMoreElements(); ) {
			current = (ROPComponentRole)(e.nextElement());
			if (current.getName().compareTo(type) == 0)	{
				roles.removeElement(current);
				return (true);
			}
		}
		return (false);
	}
    //--------------------------------------------------------------------
	public ROPComponentRole getRole(String type) {
		ROPComponentRole current;
		for (Enumeration e = roles.elements(); e.hasMoreElements(); ) {
			current = (ROPComponentRole)(e.nextElement());
			if (current.getName().compareTo(type) == 0)	return (current);
		}
		return (null);
	}
    //--------------------------------------------------------------------
	public Enumeration getRoles() {
		return (roles.elements());
	}
    //--------------------------------------------------------------------

} // End ROPComponentCore

//========================================================================

// Class ROPComponentRole

// Stores a reference to the decorated component core.
// Implements the component interface by forwarding requests to its 
// core attribute.

class ROPComponentRole implements ROPComponent {

    // Instance variables ////////////////////////////////////////////////

	private ROPComponent		target;
	private String				name;

    // Constructor ///////////////////////////////////////////////////////

    //--------------------------------------------------------------------
	public ROPComponentRole() {
		target = null;
		name = "";
    }
    //--------------------------------------------------------------------

    // Instance methods //////////////////////////////////////////////////
    
	//--------------------------------------------------------------------
	public void setTarget(ROPComponent owner) {
		target = owner;
		return;
	}
	//--------------------------------------------------------------------
	public ROPComponent getTarget() {
		return (target);
	}
	//--------------------------------------------------------------------
	public String getName() {
		return (name);
	}
	//--------------------------------------------------------------------
	public void setName(String n) {
		name = n;
		return;
	}
	//--------------------------------------------------------------------
	public boolean addRole(String type) {
		return (target.addRole(type));
	}
    //--------------------------------------------------------------------
	public boolean hasRole(String type) {
		return (target.hasRole(type));
	}
    //--------------------------------------------------------------------
	public boolean removeRole(String type) {
		return (target.removeRole(type));
	}
    //--------------------------------------------------------------------
	public ROPComponentRole getRole(String type) {
		return (target.getRole(type));
	}
    //--------------------------------------------------------------------
	public Enumeration getRoles() {
		return (target.getRoles());
	}
    //--------------------------------------------------------------------

} // End ROPComponentRole

//========================================================================
//========================================================================

// RolePattern.Java

//========================================================================
//
//      Role Object Pattern Example - March, 2000
//      Jorge Andres Diaz Pace
//
//========================================================================

// RolePatternExample.Java

//========================================================================

import java.util.Vector;
import java.util.Enumeration;

//========================================================================
//========================================================================

// Class Company 

// Implements a company (component core) that performs different 
// transactions in a market.

class Company extends ROPComponentCore {

	// Constants /////////////////////////////////////////////////////////

	private static final int DEFAULT_ITEMS = 30;

    // Instance variables ////////////////////////////////////////////////

	private int		items;
	private Market	myMarket;
	private String	name;
	private boolean performingOperation;

	private Object	itemsLock = new Object[0];
	private Object	operationLock = new Object[0];

    // Constructor ///////////////////////////////////////////////////////

    //--------------------------------------------------------------------
	public Company(String s) {
		super();
		name = s;
		items = Company.DEFAULT_ITEMS;
		myMarket = null;
		performingOperation = false;
    }
    //--------------------------------------------------------------------

    // Instance methods //////////////////////////////////////////////////
    
	//--------------------------------------------------------------------
	public String getName() {
		return (name);
	}
	//--------------------------------------------------------------------
	public void storeItems(int amount) {
		synchronized(itemsLock) {
			items = items + amount;
		}
		return;
	}
	//--------------------------------------------------------------------
	public void loadItems(int amount) {
		synchronized(itemsLock) {
			items = items - amount;
			if (items < 0) items = 0;
		}
		return;
	}
	//--------------------------------------------------------------------
	public int getItems() {
		synchronized(itemsLock) {
			return (items);
		}
	}
	//--------------------------------------------------------------------
	public void setMarket(Market m) {
		myMarket = m;
		return;
	}
	//--------------------------------------------------------------------
	public Market getMarket() {
		return (myMarket);
	}
	//--------------------------------------------------------------------
	public void setPerformingOperation(boolean performing) {
		synchronized(operationLock) {
			performingOperation = performing;
		}
		return;
	}
	//--------------------------------------------------------------------
	public boolean isPerformingOperation() {
		synchronized(operationLock) {
			return (performingOperation);
		}
	}
	//--------------------------------------------------------------------

} // End Company

//========================================================================

// Class Buyer

// Defines a role played by the company in sale transactions.
// Implements the buyer role in a contract-net protocol.

class Buyer extends ROPComponentRole implements Runnable {

	// Constants /////////////////////////////////////////////////////////

	private static final double DEFAULT_PURCHASING_PRICE	= 3.0;

	private static final int WAITING_FOR_BIDD				= 0;
	private static final int PROCESSING_PURCHASE			= 1;

    // Instance variables ////////////////////////////////////////////////

	private double	price;
	private int		currentState;
	private Seller	currentSeller;

	private Object	stateLock = new Object[0];

    // Constructor ///////////////////////////////////////////////////////

    //--------------------------------------------------------------------
	public Buyer() {
		super();
		price = Buyer.DEFAULT_PURCHASING_PRICE;
		currentState = Buyer.WAITING_FOR_BIDD;
		currentSeller = null;
    }
    //--------------------------------------------------------------------

    // Instance methods //////////////////////////////////////////////////
    
	//--------------------------------------------------------------------
	public Company getCompany() {
		return (Company)(this.getTarget());
	}
	//--------------------------------------------------------------------
	private Market getMarket() {
		Company myCompany  = (Company)(this.getTarget());
		return (myCompany.getMarket());
	}
	//--------------------------------------------------------------------
	private double getPurchasingPrice() {
		return (price);
	}
	//--------------------------------------------------------------------
	private void storeItems(int amount) {
		this.getCompany().storeItems(amount);
		return;
	}
	//--------------------------------------------------------------------
	private boolean acceptBidd(double salePrice,int amount,Seller seller) {
		if (seller.getCompany().getName().compareTo(this.getCompany().getName()) == 0)		
			return (false);
		if (salePrice > this.getPurchasingPrice())
			return (false);
		return (true);
	}
	//--------------------------------------------------------------------
	private void sendCommitment(double salePrice,int amount) {
		MarketEvent e = new MarketEvent(MarketEvent.COMMITMENT,salePrice,amount,
										currentSeller.getCompany(),this.getCompany());
		this.getMarket().dispatchEvent(e);
		return;
	}
	//--------------------------------------------------------------------
	private void sendConfirmation(double salePrice,int amount) {
		MarketEvent e = new MarketEvent(MarketEvent.CONFIRMATION,salePrice,amount,
										currentSeller.getCompany(),this.getCompany());
		this.getMarket().dispatchEvent(e);
		return;
	}
	//--------------------------------------------------------------------
	public void newBidd(double salePrice,int amount,Seller seller) {
		synchronized (stateLock) {
			if (currentState != Buyer.WAITING_FOR_BIDD)		return;
			if ( (this.acceptBidd(salePrice,amount,seller)) && 
				 (!this.getCompany().isPerformingOperation()) ) {
				this.getCompany().setPerformingOperation(true);
				currentSeller = seller;
				currentState = Buyer.PROCESSING_PURCHASE;
				this.sendCommitment(salePrice,amount);
			}
		}
		return;
	}
	//--------------------------------------------------------------------
	public void newResponse(double salePrice,int amount,boolean okPurchasing) {
		synchronized (stateLock) {
			if (currentState != Buyer.PROCESSING_PURCHASE) return;
			if (okPurchasing) {
				this.storeItems(amount);
				this.sendConfirmation(salePrice,amount);
				this.printTransaction(this,currentSeller,salePrice,amount);
			}
			this.getCompany().setPerformingOperation(false);
			currentSeller = null;
			currentState = Buyer.WAITING_FOR_BIDD;
		}
		return;
	}
	//--------------------------------------------------------------------
	private void printTransaction(Buyer buyer,Seller seller,double price,int amount) {
		System.out.print("Sale ==> ");
		System.out.print(amount+" items ");
		System.out.print(" $ "+price+" ");
		System.out.print(buyer.getCompany().getName()+" [ Buyer ] "+buyer.getCompany().getItems()+" items");
		System.out.print(" ");
		System.out.print(seller.getCompany().getName()+" [ Seller ] "+seller.getCompany().getItems()+" items");
		System.out.println();
		return;
	}
	//--------------------------------------------------------------------
	public void run() {
		return;
	}
	//--------------------------------------------------------------------
	
} // End Buyer

//========================================================================

// Class Seller

// Defines a role played by the company in sale transactions.
// Implements the seller role in a contract-net protocol.

class Seller extends ROPComponentRole implements Runnable {

	// Constants /////////////////////////////////////////////////////////

	private static final double DEFAULT_SELLING_PRICE	= 2;
	private static final int	MAX_SELLING_AMOUNT		= 10;
	
	private static final int SENDING_BIDD			= 0;
	private static final int PROCESSING_SALE		= 1;

    // Instance variables ////////////////////////////////////////////////

	private double	price;
	private int		currentState;
	private Buyer	currentBuyer;

	private Object	stateLock = new Object[0];

    // Constructor ///////////////////////////////////////////////////////

    //--------------------------------------------------------------------
	public Seller() {
		super();
		price = Seller.DEFAULT_SELLING_PRICE;
		currentState = Seller.SENDING_BIDD;
		currentBuyer = null;
    }
    //--------------------------------------------------------------------

    // Instance methods //////////////////////////////////////////////////
    
	//--------------------------------------------------------------------
	public Company getCompany() {
		return (Company)(this.getTarget());
	}
	//--------------------------------------------------------------------
	private Market getMarket() {
		Company myCompany  = (Company)(this.getTarget());
		return (myCompany.getMarket());
	}
	//--------------------------------------------------------------------
	private double getSellingPrice() {
		return (price);
	}
	//--------------------------------------------------------------------
	private int getItems() {
		return (this.getCompany().getItems());
	}
	//--------------------------------------------------------------------
	private void loadItems(int amount) {
		this.getCompany().loadItems(amount);
		return;
	}
	//--------------------------------------------------------------------
	private void sendResponse(double purchasePrice,int amount,Buyer buyer,
														boolean okOperation) {
		MarketEvent e; 
		if (okOperation)	
			e = new MarketEvent(MarketEvent.RESPONSE_OK,purchasePrice,amount,
										buyer.getCompany(),this.getCompany());
		else				
			e = new MarketEvent(MarketEvent.RESPONSE_NO,purchasePrice,amount,
										buyer.getCompany(),this.getCompany());
		this.getMarket().dispatchEvent(e);
		return;
	}
	//--------------------------------------------------------------------
	private void sendBidd() {
		synchronized (stateLock) {
			if (currentState != Seller.SENDING_BIDD) return;
			int amount = Seller.MAX_SELLING_AMOUNT;
			if (this.getItems() < Seller.MAX_SELLING_AMOUNT)	amount = this.getItems();		
			amount = (int)(Math.round(Math.random()*amount));
			if (amount == 0)	amount++;
			MarketEvent e = new MarketEvent(MarketEvent.BIDD,this.getSellingPrice(),amount,
																	null,this.getCompany());
			this.getMarket().dispatchEvent(e);
		}
		return;
	}
	//--------------------------------------------------------------------
	public void newCommitment(double purchasePrice,int amount,Buyer buyer) {
		synchronized (stateLock) {
			if (currentState != Seller.SENDING_BIDD) 
				this.sendResponse(purchasePrice,amount,buyer,false);
			else if ( (this.getCompany().getItems() >= amount) && 
					  (!this.getCompany().isPerformingOperation()) ) {
					this.getCompany().setPerformingOperation(true);
					this.loadItems(amount);
					currentBuyer = buyer;
					this.sendResponse(purchasePrice,amount,buyer,true);
					currentState = Seller.PROCESSING_SALE;
			}
			else 	this.sendResponse(purchasePrice,amount,buyer,false);
			return;
		}
	}
	//--------------------------------------------------------------------
	public void newConfirmation() {
		synchronized (stateLock) {
			if (currentState != Seller.PROCESSING_SALE) return;
			currentBuyer = null;
			currentState = Seller.SENDING_BIDD;
			this.getCompany().setPerformingOperation(false);
		}
		return;
	}
	//--------------------------------------------------------------------
	public void run() {
		this.sendBidd();
		return;
	}
	//--------------------------------------------------------------------

} // End Seller

//========================================================================

// Class Producer

// Defines a role played by the company to produce new items.

class Producer extends ROPComponentRole implements Runnable {

	// Constants /////////////////////////////////////////////////////////

	private static final int DEFAULT_RATE	= 10;

    // Instance variables ////////////////////////////////////////////////

	private int	amountRate;

    // Constructor ///////////////////////////////////////////////////////

    //--------------------------------------------------------------------
	public Producer() {
		super();
		amountRate = Producer.DEFAULT_RATE;
    }
    //--------------------------------------------------------------------

    // Instance methods //////////////////////////////////////////////////

	//--------------------------------------------------------------------
	public Company getCompany() {
		return (Company)(this.getTarget());
	}
	//--------------------------------------------------------------------
	private void produceItems() {
		if (this.getCompany().isPerformingOperation())	return;
		this.getCompany().setPerformingOperation(true);
		int amount = (int)(Math.round(Math.random()*amountRate));
		if (amount < 0) amount++;
		this.getCompany().storeItems(amount);
		System.out.print(this.getCompany().getName()+" [ Producer ] = ");
		System.out.println(" (+) "+amount+"  Items = "+this.getCompany().getItems());
		this.getCompany().setPerformingOperation(false);
		return;
	}
	//--------------------------------------------------------------------
	public void run() {
		this.produceItems();
		return;
	}
	//--------------------------------------------------------------------
	
} // End Producer

//========================================================================

// Class Market 

// It is the market where transactions between companies take place.

class Market implements Runnable {

    // Instance variables ////////////////////////////////////////////////

	private Vector	companies;
	private Vector	incomingEvents;

	private Object	eventsLock = new Object[0];

    // Constructor ///////////////////////////////////////////////////////

    //--------------------------------------------------------------------
	public Market() {
		super();
		companies = new Vector();
		incomingEvents = new Vector();
    }
    //--------------------------------------------------------------------

    // Instance methods //////////////////////////////////////////////////
    
	//--------------------------------------------------------------------
	public void addCompany(Company c) {
		if (!companies.contains(c))	{
			companies.addElement(c);
			c.setMarket(this);
		}
		return;
	}
	//--------------------------------------------------------------------
	public void dispatchEvent(MarketEvent e) {
		synchronized (eventsLock) {
			incomingEvents.addElement(e);
//			e.print();
		}
		return;
	}
	//--------------------------------------------------------------------
	private void processEvent(MarketEvent e) {
		Buyer buyer;
		Seller seller;
		switch (e.getType()) {
			case MarketEvent.BIDD:
				seller = (Seller)(e.getSource().getRole("Seller"));
				Company temp;
				for (Enumeration en = companies.elements(); en.hasMoreElements(); ) {
					temp = (Company)(en.nextElement());
					if (temp.hasRole("Buyer")) {
						buyer = (Buyer)(temp.getRole("Buyer"));
						buyer.newBidd(e.getPrice(),e.getAmount(),seller);
					}
				}
				break;
			case MarketEvent.COMMITMENT: 
				buyer = (Buyer)(e.getSource().getRole("Buyer"));
				seller = (Seller)(e.getDestination().getRole("Seller"));
				seller.newCommitment(e.getPrice(),e.getAmount(),buyer);
				break;
			case MarketEvent.RESPONSE_OK: 
				seller = (Seller)(e.getSource().getRole("Seller"));
				buyer = (Buyer)(e.getDestination().getRole("Buyer"));
				buyer.newResponse(e.getPrice(),e.getAmount(),true);
				break;
			case MarketEvent.RESPONSE_NO: 
				seller = (Seller)(e.getSource().getRole("Seller"));
				buyer = (Buyer)(e.getDestination().getRole("Buyer"));
				buyer.newResponse(e.getPrice(),e.getAmount(),false);
				break;
			case MarketEvent.CONFIRMATION:
				buyer = (Buyer)(e.getSource().getRole("Buyer"));
				seller = (Seller)(e.getDestination().getRole("Seller"));
				seller.newConfirmation();
				break;
		}
		return;
	}
	//--------------------------------------------------------------------
	public void run() {
		while (incomingEvents.size() > 0) {
			MarketEvent e;
			synchronized (eventsLock) {
				e = (MarketEvent)(incomingEvents.firstElement());
				incomingEvents.removeElementAt(0);
			}
			this.processEvent(e);
		}
		return;
	}
	//--------------------------------------------------------------------

} // End Market

//========================================================================

// Class MarketEvent

// Notification sent by the companies to the market

class MarketEvent {

	// Constants /////////////////////////////////////////////////////////

	public static final int BIDD			= 0;
	public static final int COMMITMENT		= 1;
	public static final int RESPONSE_OK		= 2;
	public static final int RESPONSE_NO		= 3;
	public static final int CONFIRMATION	= 4;

    // Instance variables ////////////////////////////////////////////////

	private int			type;
	private double		price;
	private int			amount;
	private Company		destination;
	private Company		source;

    // Constructor ///////////////////////////////////////////////////////

    //--------------------------------------------------------------------
	public MarketEvent(int type,double price,int amount,Company to,Company from) {
		this.type = type;
		this.price = price;
		this.amount = amount;
		source = from;
		destination = to;
    }
    //--------------------------------------------------------------------

    // Instance methods //////////////////////////////////////////////////
    
	//--------------------------------------------------------------------
	public int getType() {
		return (type);
	}
	//--------------------------------------------------------------------
	public int getAmount() {
		return (amount);
	}
	//--------------------------------------------------------------------
	public double getPrice() {
		return (price);
	}
	//--------------------------------------------------------------------
	public Company getSource() {
		return (source);
	}
	//--------------------------------------------------------------------
	public Company getDestination() {
		return (destination);
	}
	//--------------------------------------------------------------------
	public void print() {
		System.out.print("MarketEvent ");
		if (type == MarketEvent.BIDD)			System.out.print("Bidd");
		if (type == MarketEvent.COMMITMENT)		System.out.print("Commitment");
		if (type == MarketEvent.RESPONSE_OK)	System.out.print("ResponseOk");
		if (type == MarketEvent.RESPONSE_NO)	System.out.print("ResponseNo");
		if (type == MarketEvent.CONFIRMATION)	System.out.print("Confirmation");
		System.out.println(" "+source.getName());
	}
	//--------------------------------------------------------------------

} // End MarketEvent

//========================================================================

// Class DelayedThread

class DelayedThread extends Thread {

	// Constants /////////////////////////////////////////////////////////

	private static final long DEFAULT_FREQUENCY = 1000;

    // Instance variables ////////////////////////////////////////////////

	protected Runnable target;
	protected boolean random;
	protected long frequency;

    // Constructor ///////////////////////////////////////////////////////

    //--------------------------------------------------------------------
    public DelayedThread(Runnable ag,boolean mode,long freq) {
		super();
		target = ag;
		random = mode;
		frequency = freq;
    }
    //--------------------------------------------------------------------
    public DelayedThread(Runnable ag) {
		super();
		target = ag;
		random = true;
		frequency = DelayedThread.DEFAULT_FREQUENCY;
    }
    //--------------------------------------------------------------------

    // Instance methods //////////////////////////////////////////////////

    //--------------------------------------------------------------------
	public void run() {
	    long delay;
        while (true) {
            if (random)		delay = Math.round(frequency*Math.random());
			else			delay = Math.round(frequency);
            try { this.sleep(delay);
            } catch (InterruptedException e) {;}
	        target.run();
        }
	}
    //--------------------------------------------------------------------

} // End class DelayedThread

//========================================================================

class Example {

    // Class methods /////////////////////////////////////////////////////

    //--------------------------------------------------------------------
	private static void launchComponent(ROPComponentCore component) {
		Runnable role;
		Thread thread;
		for (Enumeration e = component.getRoles(); e.hasMoreElements(); ) {
			role = (Runnable)(e.nextElement());
			thread = new DelayedThread(role);
			thread.start();
		}
		return;
	}
	//--------------------------------------------------------------------
	private static void launchMarket(Market market) {
		Thread thread = new DelayedThread((Runnable)(market));
		thread.start();
		return;
	}
	//--------------------------------------------------------------------
	public static void main(String[] args) {
		
		// IBM
		Company ibm = new Company("IBM");
		ibm.addRole("Buyer");
		ibm.addRole("Seller");
		
		// Sun
		Company sun = new Company("SUN");
		sun.addRole("Seller");
		sun.addRole("Buyer");
		sun.addRole("Producer");

		// Apple
		Company apple = new Company("APPLE");
		apple.addRole("Seller");
		apple.addRole("Producer");

		// Market
		Market pcMarket = new Market();
		pcMarket.addCompany(ibm);
		pcMarket.addCompany(sun);
		pcMarket.addCompany(apple);

		// Running simulation
		Example.launchMarket(pcMarket);
		Example.launchComponent(ibm);
		Example.launchComponent(sun);
		Example.launchComponent(apple);

		return; 
	}
    //--------------------------------------------------------------------

} // End Example

//========================================================================
//========================================================================

// RolePatternExample.Java
